import requests
import json

from os import path


def lang_detect(text, cookie=''):
    headers = {'User-agent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_1) AppleWebKit/537.36'
                             ' (KHTML, like Gecko) Chrome/77.0.3865.90 Safari/537.36'}
    if cookie:
        headers['Cookie'] = cookie
    try:
        resp = requests.post(
            'https://fanyi.baidu.com/langdetect',
            headers=headers,
            data={'query': text}
        )
        content = resp.content.decode('utf-8')
        result = json.loads(content)
        return result.get('lan', 'en')
    except Exception as _:
        return 'en'


def get_lang_obj():
    with open(path.join(path.dirname(__file__), '../../resources/countries.json'), 'rt') as json_file:
        return json.loads(json_file.read())


def exists_lang(lang):
    return lang in get_lang_obj()


def list_supported_lang():
    lang = get_lang_obj()

    for code in lang:
        print(code, lang[code])




from translate.youdao import Youdao
from translate.baidu import Baidu
from translate.translator import Translator

translators = ['youdao', 'baidu']


def get_translator(translator) -> Translator:
    if translator == 'youdao':
        return Youdao()
    elif translator == 'baidu':
        return Baidu()
    else:
        raise InvalidTranslatorException("Invalid translator: '%s'." % translator)


class InvalidTranslatorException(Exception):
    pass



import re
import json
import xml.etree.ElementTree as xmlElementTree

from urllib import request
from urllib import parse
from translate.util import QueryString
from translate.translator import Translator
from xml2dict import parser as xml2dict
from translate.util.formatBuilder import YoudaoFormatBuilder


class Youdao(Translator):
    def __init__(self):
        super().__init__()

        self.keyfrom = 'chrome.extension'
        self.doctype = 'xml'
        self.dogVersion = '1.0'
        self.xmlVersion = '3.2'
        self.appVer = "3.1.17.4208"
        self.client = 'deskdict'
        self.version = 2.1
        self._result = None
        self._last_query = ''

        self.api_list = {
            'fsearch': {
                'url': 'http://dict.youdao.com/fsearch',
                'data': {
                    "client": self.client,
                    "keyfrom": self.keyfrom,
                    "pos": -1,
                    "doctype": self.doctype,
                    "xmlVersion": self.xmlVersion,
                    "dogVersion": self.dogVersion,
                    "appVer": self.appVer,
                    "le": "eng",
                    "q": None,
                }
            },
            'translate': {
                'url': 'http://fanyi.youdao.com/translate',
                'data': {
                    "client": self.client,
                    "keyfrom": self.keyfrom,
                    "xmlVersion": "1.1",
                    "dogVersion": self.dogVersion,
                    "ue": "utf-8",
                    "version": self.version,
                    "doctype": self.doctype,
                    "i": None,
                    'form': 'AUTO',
                    'to': 'AUTO',
                }
            }
        }

    def do_request(self, url):
        req = request.Request(url, headers={'User-Agent': self.user_agent})
        res = request.urlopen(req)

        if res.status == 200:
            xmldom = xmlElementTree.parse(res)
            return xml2dict(xmldom)

    def fsearch(self, word):
        api = self.api_list['fsearch']

        api['data']['q'] = parse.quote(word)
        return self.do_request(api['url'] + '?' + QueryString.stringify(api['data']))

    def translate(self, text):
        api = self.api_list['translate']

        api['data']['i'] = parse.quote(text)
        return self.do_request(api['url'] + '?' + QueryString.stringify(api['data']))

    def query(self, text):
        if self._last_query != '' and self._last_query == text:
            return self

        self._last_query = text 
        chinese_regex = r'[\u4e00-\u9fa5]'
        words = re.findall(r'\w+', text)

        if re.match(chinese_regex, text):
            words = re.findall(chinese_regex, text)

        if len(words) > 4:
            self._result = self.translate(text)
        else:
            self._result = self.fsearch(text)

        return self

    def get_result(self):
        return self._result

    def serialize(self):
        if self.get_result() is None:
            return ""

        return json.dumps(self.get_result())

    def get_english_chinese(self):
        return self.get_result().get('custom-translation')

    def get_web_interpretation(self):
        return self.get_result().get('yodao-web-dict')

    def format(self, verbose=0):
        builder = YoudaoFormatBuilder()
        if 'translation' in self.get_result():
            builder.of_translation(self.get_result().get('translation'))

        builder.of_phonetic_symbol(
            self.get_result().get("us-phonetic-symbol", ""),
            self.get_result().get("uk-phonetic-symbol", "")
        )

        if self.get_english_chinese():
            builder.of_english_chinese(self.get_english_chinese())

        if self.get_web_interpretation() and verbose >= 1:
            builder.of_web_interpretation(self.get_web_interpretation().get('web-translation'))

        return builder.get_result()

    def get_name(self):
        return "Youdao"





# encoding: utf-8

__all__ = ['format_simple_means', 'format_oxford_entry', 'format_oxford_unbox',
           'format_collins', 'with_new_line', 'format_trans_result', 'format_liju_double']


def indent(level=4):
    return " " * level


def format_examples(examples):
    text = ''
    for example in examples:
        ex = replace_html_blod(example.get('ex', ''))
        tran = example.get('tran')
        # tts_mp3 = example.get('tts_mp3', '')
        # tts_size = example.get('tts_size', '')

        text += "%s\n" % ex
        text += '\033[04m%s\033[0m\n\n' % tran
        # text += 'mp3 url: \033[04m%s\033[0m, mp3 size: %s\n\n' % ("https://fanyi.baidu.com/" + tts_mp3, tts_size)

    return text


def replace_html_blod(text):
    replace_prefix = text.replace('<b>', '\033[1m')
    replace_suffix = replace_prefix.replace('</b>', '\033[0m')

    return replace_suffix


def with_new_line(text):
    if text is None or text == '':
        return text
    return text + '\n'


def with_italic(text):
    return "\033[2m%s\033[0m" % text


def bold_title(title):
    return "\033[1m%s\033[0m" % title


def format_simple_means(result):
    simple_means = result.get('simple_means')

    if simple_means is None:
        return ''

    symbols = simple_means.get('symbols')
    text = bold_title('简明释义') + "\n"

    def format_means(means):
        s = ''
        for mean in means:
            if type(mean) is dict:
                word_mean = mean.get('word_mean', '')
                inner_means = mean.get('means', [])

                s += word_mean + ' ' + "; ".join(inner_means) + '\n'
            elif type(mean) is str:
                s += mean + ';'
        return s

    for symbol in symbols:
        parts = symbol.get('parts', '')
        ph_am = symbol.get('ph_am', '')
        ph_en = symbol.get('ph_en')
        
        text += "%s %s\n" % (ph_am, ph_en)
        for part in parts:
            line = part.get('part', '') + format_means(part.get('means'))
            text += line + "\n"

    return text


def format_oxford_entry(oxford):
    if type(oxford) is not dict or 'entry' not in oxford:
        return ""

    entrys = oxford.get('entry')
    text = bold_title('牛津词典:') + "\n"

    for entry in entrys:
        entry_name = entry.get('name')
        text += entry_name + "\n"

        entry_data_list = entry.get('data')
        for entry_data in entry_data_list:
            data = entry_data.get('data', []) 
            ng_data = filter(lambda item: item.get('tag') == 'n-g', data)

            for i, ng in enumerate(ng_data):
                ng_details = ng.get('data', [])

                for j, ng_detail in enumerate(ng_details):
                    if 'chText' not in ng_detail:
                        continue
                    ch_text = ng_detail.get('chText', '')
                    en_text = ng_detail.get('enText', '')

                    if j == 0:
                        ch_text = "%d.%s" % (i + 1, ch_text)
                        en_text = "%s" % en_text

                    else:
                        ch_text = "\033[2m%s\033[0m" % ch_text
                        en_text = "\033[2m%s\033[0m\n" % en_text

                    text += ch_text + "\n"
                    text += en_text + "\n"
    return text


def format_oxford_unbox(oxford):
    if type(oxford) is not dict or 'unbox' not in oxford:
        return ""

    text = ""
    unbox = oxford.get('unbox')

    for box in unbox:
        box_type = box.get('type')
        box_data = box.get('data', [])

        if box_type == 'more_about':
            text += bold_title('补充说明') + "\n"

        for datum in box_data:
            if datum.get('tag') == 'title':
                text += datum.get('text', '') + '\n'
            else:
                text_data = datum.get('data', [])
                for i, text_datum in enumerate(text_data):
                    if type(text_datum) is str:
                        text += text_datum
                    elif 'data' in text_datum:
                        for j, datum in enumerate(text_datum.get('data', [])):
                            text += format_oxford_tag(datum, j)
                    else:
                        text += format_oxford_tag(text_datum, i)
    return text


def format_oxford_tag(datum, i):
    text = ''
    ch_text = datum.get('chText', '')
    en_text = "* " + datum.get('enText', '')

    if i > 0:
        ch_text = "\033[02m%s\033[0m" % ch_text
        en_text = "\033[02m%s\033[0m" % en_text

    text += en_text + '\n'
    text += ch_text + '\n\n'

    return text


def format_collins(collins):
    text = bold_title('柯林斯词典:') + '\n'

    if collins is None:
        return ""

    if 'menus' in collins:
        menus = collins.get('menus', [])

        for menu in menus:
            text += format_entrys(menu.get('entry', []))
    if 'entry' in collins:
        text += format_entrys(collins.get('entry', []))

    return text

                        
def format_entrys(entry):
    if not entry:
        return ""
    text = ''

    for n, item in enumerate(entry):
        if item.get('type') != 'mean':
            continue

        values = item.get('value', [])
        for value in values:
            define = value.get('def')
            trans = value.get('tran')
            means = value.get('mean_type')

            posp = " ".join(map(lambda item: item.get('label'), value.get('posp', [])))

            text += with_new_line("%s. %s %s" % (with_italic(str(n + 1)), posp, trans))
            text += with_new_line(replace_html_blod(define))

            for mean in means:
                info_type = mean.get('info_type')
                
                if info_type == 'example':
                    examples = mean.get(info_type, [])
                    text += format_examples(examples)

                if info_type == 'posc':
                    posc = mean.get(info_type)
                    for pos in posc:
                        examples = pos.get('example')
                        define = pos.get('def', '')
                        text += with_new_line(replace_html_blod(define))

                        text += format_examples(examples)
    return text


def format_trans_result(result):
    if not result:
        return ''

    text = ''
    data = result.get('data', [])
    keywords = result.get('keywords', [])

    for datum in data:
        text += datum.get('dst') + '\n'

    if len(keywords) > 0:
        text += bold_title('重点词汇:') + '\n'

    for keyword in keywords:
        text += keyword.get('word', '') + ': ' + "; ". join(keyword.get('means', [])) + '\n' 

    return text


def format_liju_double(double_data):
    text = ""

    for data in double_data:
        ch_text_list, en_text_list, provider = data[0:3]
        ch_text = " ".join(map(lambda item: item[0], ch_text_list))
        en_text = " ".join(map(lambda item: item[0], en_text_list))

        text += with_new_line(ch_text)
        text += with_new_line(en_text)

        if provider:
            text += '\033[2m' + provider + '\033[0m\n\n'

    if text != "":
        text = bold_title("双语例句:\n\n") + text

    return text





class QueryString:
    @staticmethod
    def parse(query_string):
        result = {}
        for query in query_string.split("&"):
            key_value = query.split("=")
            if len(key_value) == 2:
                key, value = key_value
                result[key] = value
        return result

    @staticmethod
    def stringify(query_dict: dict) -> str:
        result_list = []
        for key in query_dict:
            result_list.append("%s=%s" % (key, query_dict[key]))
        return "&".join(result_list)


def get_plain_text(text):
    """
    clear \n and left space
    :param text: input text
    :type text str
    :return: str
    """
    if not text or not isinstance(text, str):
        return ""

    return text.lstrip().replace("\n", "").replace("\r", "")



from functools import lru_cache
from .factory import get_translator
from .util import get_plain_text
from .translator import Translator

__all__ = ['Translator', 'get_plain_text', 'get_translator', 'console', '__version__']
__version__ = '0.0.2'


class ConsoleCommand(object):
    def __init__(self):
        self._commands = {}

    def add_command(self, command, help):
        self._commands[command] = help
        return self

    def print_help(self):
        print('available commands:')
        for command in self._commands:
            print('    ' + command + '%40s' % self._commands[command])


@lru_cache()
def translate(text, translator: Translator, verbose):
    return translator.query(text).format(verbose)


def console(translator: Translator, verbose=0):
    prompt = '[%s] >>> ' % translator.get_name()
    command = ConsoleCommand()
    command.\
        add_command('@quit', 'quit the console').\
        add_command('@cache:info', 'Show the lru cache info.').\
        add_command('@cache:clear', 'Clear the lru cache info')
    while True:
        text = input(prompt)
        if text == '@quit':
            break
        elif text == '@help':
            command.print_help()
        elif text == '@cache:info':
            print(translate.cache_info())
        elif text == '@cache:clear':
            translate.cache_clear()
            print('cache cleared.')
        elif not text:
            continue
        else:
            print(translate(text, translator, verbose))



# encoding: utf-8
import re
import requests
import json

from translate.translator import Translator
from translate.util.baidu_sign import generate_token

from translate.util.formatBuilder import BaiduFormatBuilder


class Baidu(Translator):
    def __init__(self):
        super().__init__()

        self.gtk = ""
        self.token = ""
        self._last_query = ""

        self.api = "https://fanyi.baidu.com/v2transapi"
        self.api_home = "https://fanyi.baidu.com/"
        self.api_lan_detect = 'https://fanyi.baidu.com/langdetect'

        self._result = None
        self._response = None

        self.session = requests.session()
        self.session.headers = {'User-Agent': self.user_agent, 'x-requested-with': 'XMLHttpRequest'}

    def set_cookie(self, cookie):
        self.session.headers['Cookie'] = cookie

    def get_param(self, query):
        param = {
            "from": self.trans_from,
            "to": self.trans_to,
            "query": query,
            "transtype": "translang",
            "simple_means_flag": 3,
            "sign": self.sign(query),
            "token": self.token
        }

        return param

    def get_token_and_gtk(self):
        resp = self.session.get(self.api_home)
        content = resp.content.decode('utf-8')
        token = re.findall(r'token: (.*)', content)[0]
        gtk = re.findall(r'gtk = (.\d+\.\d+.)', content)[0]

        gtk = gtk.replace("'", "")
        token = token.replace("'", "").replace(',', '')

        return gtk, token

    def sign(self, text):
        if not self.token or not self.gtk:
            self.gtk, self.token = self.get_token_and_gtk()

        return generate_token(text, self.gtk)

    def query(self, text):
        if self._last_query == text:
            return self

        self._last_query = text
        param = self.get_param(text)
        resp = self.session.post(self.api, data=param)

        content = resp.content.decode('utf-8')
        self._result = json.loads(content)
        self._response = resp

        return self

    def get_result(self):
        return self._result

    def serialize(self):
        return json.dumps(self.get_result())

    def __str__(self):
        return self.format(6)

    def format(self, verbose=0):
        result = self.get_result()
        if result is None:
            return ""

        dict_result = result.get('dict_result', {})
        oxford = dict_result.get('oxford')
        collins = dict_result.get('collins')
        liju_result = result.get('liju_result', {})
        double_str = liju_result.get('double')

        builder = BaiduFormatBuilder()

        builder.of_simple_means(dict_result)

        if verbose >= 1:
            builder.of_oxford_entry(oxford)

        if verbose >= 2:
            builder.of_oxford_unbox(oxford)

        if verbose >= 3:
            builder.of_collins(collins)

        if verbose >= 4 and double_str:
            builder.of_double(json.loads(double_str))

        if 'dict_result' not in result:
            builder.of_trans_result(result.get('trans_result'))

        return builder.get_result()

    def get_name(self):
        return "Baidu"



import math

__all__ = ['generate_token']


def encode_by_key(num, key):
    t = 0
    while t < len(key) - 2:
        a = key[t + 2]
        if ord(a) >= ord('a'):
            a = ord(a) - 87
        else:
            a = int(a)

        if key[t + 1] == '+':
            a = num >> a
        else:
            a = num << a

        if '+' == key[t]:
            num = num + a & 4294967295
        else:
            num = num ^ a

        t += 3
    return num


def substr(string, start, length):
    end = start + length
    if start < 0:
        end = length
    return string[start:end]


def generate_token(text, gtk):
    if len(text) > 30:
        text = substr(text, 0, 10) + substr(text, math.floor(len(text) / 2) - 5, 10) + substr(text, -10, len(text))

    gtk_list = gtk.split('.')
    prefix = int(gtk_list[0])
    suffix = int(gtk_list[1])
    s = []
    v = 0

    while v < len(text):
        a = ord(text[v])

        if 128 > a:
            s.append(a)
        else:
            if 2048 > a:
                s.append(a >> 6 | 192)
            else:
                if 55296 == (64512 & a) and v + 1 < text.length and 56320 == (64512 & ord(text[v + 1])):
                    a = 65536 + ((1024 & a) << 10) + (1023 & ord(text[++v]))
                    s.append(a >> 18 | 240)
                    s.append(a >> 12 & 63 | 128)
                else:
                    s.append(a >> 12 | 224)

                s.append(a >> 6 & 63 | 128)

            s.append(63 & a | 128)
        v += 1

    key1 = '+-a^+6'
    key2 = "+-3^+b+-f"
    p = prefix

    b = 0
    while b < len(s):
        p += s[b]
        p = encode_by_key(p, key1)
        b += 1

    p = encode_by_key(p, key2)
    p ^= suffix

    if p < 0:
        p = (2147483647 & p) + 2147483648

    p %= 1e6
    p = int(p)

    return str(p) + '.' + str(p ^ prefix)
